'use strict';
/*----------------------------------------------------------------------------------------------**
 * Based on and Modified from Haraka's tls_socket.js class (https://github.com/haraka/Haraka)
 *----------------------------------------------------------------------------------------------*/

var util = require('util');
var stream = require('stream');
var tls = require('tls');

function tlsWrapper(socket) {
	stream.Stream.call(this);
	this.readable = this.writable = true;
	this._timeout = 0;
	this._keepalive = false;
	this._writeState = true;
	this._pending = [];
	this._pendingCallbacks = [];
	if (socket)
		this.attach(socket);
}
util.inherits(tlsWrapper, stream.Stream);

tlsWrapper.prototype.startTLS = function(options) {
	var self = this;
	var socket = self;
	var netSocket = self.targetsocket;

	socket.clean();

	var tlsSocket = new tls.TLSSocket(netSocket, options);
	/*
	 * var secureContext = tls.createSecureContext(options); var tlsSocket = new tls.TLSSocket(netSocket, {
	 * secureContext : secureContext, isServer : options.isServer, // server : options.server, requestCert :
	 * options.requestCert, rejectUnauthorized : options.rejectUnauthorized, requestOCSP : options.requestOCSP,
	 * SNICallback : options.SNICallback });
	 */

	self.attach(tlsSocket);
}

tlsWrapper.prototype.pause = function() {
	if (this.targetsocket.pause) {
		this.targetsocket.pause();
		this.readable = false;
	}
};

tlsWrapper.prototype.resume = function() {
	if (this.targetsocket.resume) {
		this.readable = true;
		this.targetsocket.resume();
	}
};

tlsWrapper.prototype.attach = function(socket) {
	var self = this;
	self.targetsocket = socket;
	self.targetsocket.on('data', function(data) {
		self.emit('data', data);
	});
	self.targetsocket.on('connect', function(a, b) {
		self.emit('connect', a, b);
	});
	self.targetsocket.on('secureConnection', function(a, b) {
		self.emit('secureConnection', a, b);
		self.emit('secure', a, b);
	});
	self.targetsocket.on('secure', function(a, b) {
		self.emit('secureConnection', a, b);
		self.emit('secure', a, b);
	});
	self.targetsocket.on('end', function() {
		self.writable = self.targetsocket.writable;
		self.emit('end');
	});
	self.targetsocket.on('close', function(had_error) {
		self.writable = self.targetsocket.writable;
		self.emit('close', had_error);
	});
	self.targetsocket.on('drain', function() {
		self.emit('drain');
	});
	self.targetsocket.on('error', function(exception) {
		self.writable = self.targetsocket.writable;
		self.emit('error', exception);
	});
	self.targetsocket.on('timeout', function() {
		self.emit('timeout');
	});

	if (self.targetsocket.remotePort) {
		self.remotePort = self.targetsocket.remotePort;
	}
	if (self.targetsocket.remoteAddress) {
		self.remoteAddress = self.targetsocket.remoteAddress;
	}
	if (self.targetsocket.localPort) {
		self.localPort = self.targetsocket.localPort;
	}
	if (self.targetsocket.localAddress) {
		self.localAddress = self.targetsocket.localAddress;
	}
};

tlsWrapper.prototype.clean = function(data) {
	if (this.targetsocket && this.targetsocket.removeAllListeners) {
		this.targetsocket.removeAllListeners('data');
		this.targetsocket.removeAllListeners('secureConnection');
		this.targetsocket.removeAllListeners('secure');
		this.targetsocket.removeAllListeners('end');
		this.targetsocket.removeAllListeners('close');
		this.targetsocket.removeAllListeners('error');
		this.targetsocket.removeAllListeners('drain');
	}
	this.targetsocket = {};
	this.targetsocket.write = function() {
	};
};

tlsWrapper.prototype.write = function(data, encoding, callback) {
	if (this.targetsocket.write) {
		return this.targetsocket.write(data, encoding, callback);
	}
	return false;
};

tlsWrapper.prototype.end = function(data, encoding) {
	if (this.targetsocket.end) {
		return this.targetsocket.end(data, encoding);
	}
};

tlsWrapper.prototype.destroySoon = function() {
	if (this.targetsocket.destroySoon) {
		return this.targetsocket.destroySoon();
	}
};

tlsWrapper.prototype.destroy = function() {
	if (this.targetsocket.destroy) {
		return this.targetsocket.destroy();
	}
};

tlsWrapper.prototype.setKeepAlive = function(bool) {
	this._keepalive = bool;
	return this.targetsocket.setKeepAlive(bool);
};

tlsWrapper.prototype.setNoDelay = function(/* true||false */) {
};

tlsWrapper.prototype.setTimeout = function(timeout) {
	this._timeout = timeout;
	return this.targetsocket.setTimeout(timeout);
};

exports.tlsWrapper = tlsWrapper;